@@ -36,14 +36,14 @@ void capture_destroy_window ();
 void capture_init ();
 void capture_cleanup ();
 
-gint capture_do
+gboolean capture_do
 (
     Image **p_capture_image,
     ImageInfo **p_capture_image_info,
     CaptureType cap_type,
     guint cap_delay
 );
-gint capture_preview (Image *capture_image);
+gboolean capture_preview (Image *capture_image);
 
 void capture_show ();
 
