@@ -21,9 +21,17 @@
  * save.c: Image saving routines
  */
 
+#include "config.h"
+
+#ifdef HAVE_SYS_STAT_H
+#include <sys/stat.h>
+#endif
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
 #include <stdio.h>
 #include <magick/api.h>
-#include <magick/xwindows.h>
 #include <glib.h>
 #include <gdk/gdk.h>
 #include <gtk/gtk.h>
@@ -32,7 +40,7 @@
 #include "common.h"
 #include "util.h"
 #include "stringtable.h"
-#include "widget.h"
+#include "widgproc.h"
 #include "save.h"
 
 /* Global variables */
@@ -175,6 +183,8 @@ void save_create_dialog ()
     /* Create a GTK save dialog box. */
     
     save_dialog = gtk_file_selection_new (TITLE_SAVE_WINDOW);
+    gtk_window_set_modal (GTK_WINDOW (save_dialog), TRUE);
+    
     gtk_signal_connect
     (
         GTK_OBJECT (save_dialog),
@@ -395,18 +405,6 @@ void save_create_filetype_dialog ()
         1,
         GTK_JUSTIFY_LEFT
     );
-    gtk_clist_set_column_width
-    (
-        GTK_CLIST (save_filetype_clist),
-        0,
-        75
-    );
-    gtk_clist_set_column_width
-    (
-        GTK_CLIST (save_filetype_clist),
-        1,
-        400
-    );
 
     gtk_clist_append (GTK_CLIST (save_filetype_clist), (gchar **) clist_auto);
     for (magick_index = magick_info;
@@ -419,6 +417,8 @@ void save_create_filetype_dialog ()
         gtk_clist_append (GTK_CLIST (save_filetype_clist), clist_row);
     }    
 
+    gtk_clist_columns_autosize (GTK_CLIST (save_filetype_clist));
+
     gtk_signal_connect
     (
         GTK_OBJECT (save_filetype_clist),
@@ -432,7 +432,7 @@ void save_create_filetype_dialog ()
     gtk_scrolled_window_set_policy
     (
         GTK_SCROLLED_WINDOW (scrolled_window),
-        GTK_POLICY_AUTOMATIC,
+        GTK_POLICY_NEVER,
         GTK_POLICY_ALWAYS
     );
     gtk_scrolled_window_add_with_viewport
@@ -542,7 +542,6 @@ void save_filetype_set (gchar *filetype)
 {
     GString *fmt_filetype_selected;
 
-    g_print ("File type selected: %s\n", filetype);
     filetype_selected = filetype;
     
     fmt_filetype_selected = g_string_new (NULL);
@@ -619,12 +618,15 @@ void save_do (Image *image, ImageInfo *image_info)
 
 void save_ok (GtkWidget *widget, GtkFileSelection *filesel)
 {
-    GtkWidget *progress_window;
-    GtkWidget *box;
     gchar *save_filename_text;
-    GString *save_message;
-    gint i;
-
+    
+#ifdef HAVE_STAT
+    
+    struct stat stat_buffer;
+    int stat_result;
+    
+#endif /* HAVE_STAT */
+    
     /* Retrieve the filename from the file selection box. */
     
     save_filename_text = gtk_file_selection_get_filename
@@ -633,6 +635,110 @@ void save_ok (GtkWidget *widget, GtkFileSelection *filesel)
     );
     g_string_assign (save_filename, save_filename_text);
 
+    /* Determine if the file exists. If so, pop up a confirmation box. */
+
+#ifdef HAVE_STAT
+
+    stat_result = stat (save_filename -> str, &stat_buffer);
+    if (stat_result == 0)
+    {
+        save_overwrite_dialog ();
+    }
+    else
+    {
+        save_progress (NULL, NULL);
+    }
+    
+#else
+
+    save_progress (NULL, NULL);
+    
+#endif /* HAVE_STAT */
+}
+
+/*********************************************************************
+ *
+ * save_overwrite_dialog: Open a dialog box which prompts the user
+ * whether or not to overwrite an existing image file.
+ *
+ * Parameters:
+ *   None.
+ *
+ * Return value:
+ *   None.
+ */
+
+#ifdef HAVE_STAT
+void save_overwrite_dialog ()
+{
+    GtkWidget *overwrite_dialog;
+    GString *overwrite_message;
+
+    /*
+     * Create a small dialog box which will prompt the user to
+     * determine whether an existing file should be overwritten.
+     */
+    
+    overwrite_message = g_string_new (NULL);    
+    g_string_sprintf
+    (
+        overwrite_message,
+        FMT_OVERWRITE,
+        save_filename -> str
+    );
+
+    overwrite_dialog = gtk_dialog_new ();
+    gtk_window_set_modal (GTK_WINDOW (overwrite_dialog), TRUE);
+    gtk_window_set_title (GTK_WINDOW (overwrite_dialog), TITLE_OVERWRITE);
+    gtk_container_set_border_width (GTK_CONTAINER (overwrite_dialog), 10);
+
+    add_label
+    (
+        GTK_DIALOG (overwrite_dialog) -> vbox,
+        overwrite_message -> str
+    );
+    add_button
+    (
+        GTK_DIALOG (overwrite_dialog) -> action_area,
+        GTK_OBJECT (overwrite_dialog),
+        GTK_SIGNAL_FUNC (save_progress),
+        LABEL_YES
+    );
+    add_button
+    (
+        GTK_DIALOG (overwrite_dialog) -> action_area,
+        GTK_OBJECT (overwrite_dialog),
+        GTK_SIGNAL_FUNC (gtk_widget_destroy),
+        LABEL_NO
+    );    
+        
+    gtk_widget_show (overwrite_dialog);
+    
+    g_string_free (overwrite_message, TRUE);
+}
+#endif /* HAVE_STAT */
+
+/*********************************************************************
+ *
+ * save_progress: Open a progress window indicating that the file
+ * is being saved. Proceed to save the file.
+ *
+ * Parameters:
+ *   widget - GTK widget connected to the callback function
+ *   data - callback data passed to this function
+ *
+ * Return value:
+ *   None.
+ */
+
+void save_progress (GtkWidget *widget, gpointer data)
+{
+    GtkWidget *progress_window;
+    GtkWidget *box;
+    GString *save_message;
+
+    gboolean result;
+    
     /*
      * Create a small dialog box which will display a message,
      * indicating that the image is being saved.
@@ -643,43 +749,47 @@ void save_ok (GtkWidget *widget, GtkFileSelection *filesel)
     (
         save_message,
         FMT_SAVING,
-        save_filename_text,
+        save_filename -> str,
         filetype_selected
     );
 
     progress_window = gtk_window_new (GTK_WINDOW_DIALOG);
     gtk_window_set_modal (GTK_WINDOW (progress_window), TRUE);
-    gtk_window_set_title (GTK_WINDOW (progress_window), TITLE_SAVING_FILE);
+    gtk_window_set_title (GTK_WINDOW (progress_window), save_message -> str);
     gtk_container_set_border_width (GTK_CONTAINER (progress_window), 20);
 
     box = gtk_vbox_new (FALSE, 0);
     add_label (box, save_message -> str);
     gtk_container_add (GTK_CONTAINER (progress_window), box);
-    gtk_widget_show (box);
-    gtk_widget_show (progress_window);
-
-    /*
-     * Ensure that the progress message dialog box is properly displayed.
-     * The save_image_to_file call blocks processing of GTK events, so
-     * we have to process a small set of events here. This is a temporary
-     * solution until we can make this application multithreaded.
-     */
+    gtk_widget_show_now (box);
 
     while (gtk_events_pending ())
     {
         gtk_main_iteration ();
     }
-    for (i = 0; i < 100; i++)
+    
+    gtk_widget_show_now (progress_window);
+
+    while (gtk_events_pending ())
     {
         gtk_main_iteration ();
     }
-    
-    /* Save the image. */
-    
-    save_image_to_file (save_filename_text);
 
-    gtk_widget_show (progress_window);
+    /* Save the image. */
+
+    result = save_image_to_file (save_filename -> str);
+
+    /*
+     * Delete the progress box. If the function was connected to an object,
+     * delete the object as well.
+     */
+    
     gtk_widget_destroy (progress_window);
+    if (widget != NULL)
+    {
+        gtk_widget_destroy (widget);
+    }
+    g_string_free (save_message, TRUE);
 }
 
 /*********************************************************************
@@ -713,7 +823,7 @@ gboolean save_image_to_file (gchar *save_path)
         save_image,
         save_image -> columns,
         save_image -> rows,
-        1
+        TRUE
     );
     disk_image_info = CloneImageInfo (save_image_info);
     if (disk_image == NULL || disk_image_info == NULL)
